@@ -38,10 +38,7 @@ class Part:
         self.content_sha1 = content_sha1
 
     def __repr__(self):
-        return '<{} {} {} {} {}>'.format(
-            self.__class__.__name__, self.file_id, self.part_number, self.content_length,
-            self.content_sha1
-        )
+        return f'<{self.__class__.__name__} {self.file_id} {self.part_number} {self.content_length} {self.content_sha1}>'
 
     def __eq__(self, other):
         return isinstance(other, self.__class__) and self.__dict__ == other.__dict__
