@@ -17,7 +17,7 @@ import random
 import re
 import threading
 import time
-from contextlib import contextmanager, suppress
+from contextlib import contextmanager
 from typing import Iterable
 
 from requests.structures import CaseInsensitiveDict
@@ -73,7 +73,7 @@ logger = logging.getLogger(__name__)
 
 
 def get_bytes_range(data_bytes, bytes_range):
-    """ Slice bytes array using bytes range """
+    """Slice bytes array using bytes range"""
     if bytes_range is None:
         return data_bytes
     if bytes_range[0] > bytes_range[1]:
@@ -82,7 +82,7 @@ def get_bytes_range(data_bytes, bytes_range):
         raise UnsatisfiableRange()
     if bytes_range[1] > len(data_bytes):
         raise UnsatisfiableRange()
-    return data_bytes[bytes_range[0]:bytes_range[1] + 1]
+    return data_bytes[bytes_range[0] : bytes_range[1] + 1]
 
 
 class KeySimulator:
@@ -92,8 +92,15 @@ class KeySimulator:
     """
 
     def __init__(
-        self, account_id, name, application_key_id, key, capabilities, expiration_timestamp_or_none,
-        bucket_id_or_none, bucket_name_or_none, name_prefix_or_none
+        self,
+        account_id,
+        name,
+        application_key_id,
+        key,
+        capabilities,
+        expiration_timestamp_or_none,
+        buckets_or_none,
+        name_prefix_or_none,
     ):
         self.name = name
         self.account_id = account_id
@@ -101,18 +108,23 @@ class KeySimulator:
         self.key = key
         self.capabilities = capabilities
         self.expiration_timestamp_or_none = expiration_timestamp_or_none
-        self.bucket_id_or_none = bucket_id_or_none
-        self.bucket_name_or_none = bucket_name_or_none
+        self.buckets_or_none = buckets_or_none
         self.name_prefix_or_none = name_prefix_or_none
 
+    def _get_bucket_ids(self):
+        if self.buckets_or_none is None:
+            return None
+
+        return [item['id'] for item in self.buckets_or_none]
+
     def as_key(self):
         return dict(
             accountId=self.account_id,
-            bucketId=self.bucket_id_or_none,
+            bucketIds=self._get_bucket_ids(),
             applicationKeyId=self.application_key_id,
             capabilities=self.capabilities,
-            expirationTimestamp=self.expiration_timestamp_or_none and
-            self.expiration_timestamp_or_none * 1000,
+            expirationTimestamp=self.expiration_timestamp_or_none
+            and self.expiration_timestamp_or_none * 1000,
             keyName=self.name,
             namePrefix=self.name_prefix_or_none,
         )
@@ -125,6 +137,7 @@ class KeySimulator:
         """
         result = self.as_key()
         result['applicationKey'] = self.key
+
         return result
 
     def get_allowed(self):
@@ -132,8 +145,7 @@ class KeySimulator:
         Return the 'allowed' structure to include in the response from b2_authorize_account.
         """
         return dict(
-            bucketId=self.bucket_id_or_none,
-            bucketName=self.bucket_name_or_none,
+            buckets=self.buckets_or_none,
             capabilities=self.capabilities,
             namePrefix=self.name_prefix_or_none,
         )
@@ -152,8 +164,8 @@ class PartSimulator:
             fileId=self.file_id,
             partNumber=self.part_number,
             contentLength=self.content_length,
-            contentSha1=self.content_sha1
-        )  # yapf: disable
+            contentSha1=self.content_sha1,
+        )
 
 
 class FileSimulator:
@@ -280,10 +292,12 @@ class FileSimulator:
             if self.server_side_encryption.mode == EncryptionMode.SSE_B2:
                 headers['X-Bz-Server-Side-Encryption'] = self.server_side_encryption.algorithm.value
             elif self.server_side_encryption.mode == EncryptionMode.SSE_C:
-                headers['X-Bz-Server-Side-Encryption-Customer-Algorithm'
-                       ] = self.server_side_encryption.algorithm.value
-                headers['X-Bz-Server-Side-Encryption-Customer-Key-Md5'
-                       ] = self.server_side_encryption.key.key_md5()
+                headers['X-Bz-Server-Side-Encryption-Customer-Algorithm'] = (
+                    self.server_side_encryption.algorithm.value
+                )
+                headers['X-Bz-Server-Side-Encryption-Customer-Key-Md5'] = (
+                    self.server_side_encryption.key.key_md5()
+                )
             elif self.server_side_encryption.mode in (EncryptionMode.NONE, EncryptionMode.UNKNOWN):
                 pass
             else:
@@ -291,8 +305,10 @@ class FileSimulator:
 
         if range_ is not None:
             headers['Content-Range'] = 'bytes %d-%d/%d' % (
-                range_[0], range_[0] + content_length - 1, len(self.data_bytes)
-            )  # yapf: disable
+                range_[0],
+                range_[0] + content_length - 1,
+                len(self.data_bytes),
+            )
         return headers
 
     def as_upload_result(self, account_auth_token):
@@ -308,10 +324,11 @@ class FileSimulator:
             action=self.action,
             uploadTimestamp=self.upload_timestamp,
             replicationStatus=self.replication_status and self.replication_status.value,
-        )  # yapf: disable
+        )
         if self.server_side_encryption is not None:
-            result['serverSideEncryption'
-                  ] = self.server_side_encryption.serialize_to_json_for_request()
+            result['serverSideEncryption'] = (
+                self.server_side_encryption.serialize_to_json_for_request()
+            )
         result['fileRetention'] = self._file_retention_dict(account_auth_token)
         result['legalHold'] = self._legal_hold_dict(account_auth_token)
         return result
@@ -329,10 +346,11 @@ class FileSimulator:
             action=self.action,
             uploadTimestamp=self.upload_timestamp,
             replicationStatus=self.replication_status and self.replication_status.value,
-        )  # yapf: disable
+        )
         if self.server_side_encryption is not None:
-            result['serverSideEncryption'
-                  ] = self.server_side_encryption.serialize_to_json_for_request()
+            result['serverSideEncryption'] = (
+                self.server_side_encryption.serialize_to_json_for_request()
+            )
         result['fileRetention'] = self._file_retention_dict(account_auth_token)
         result['legalHold'] = self._legal_hold_dict(account_auth_token)
         return result
@@ -353,10 +371,11 @@ class FileSimulator:
             fileInfo=self.file_info,
             uploadTimestamp=self.upload_timestamp,
             replicationStatus=self.replication_status and self.replication_status.value,
-        )  # yapf: disable
+        )
         if self.server_side_encryption is not None:
-            result['serverSideEncryption'
-                  ] = self.server_side_encryption.serialize_to_json_for_request()
+            result['serverSideEncryption'] = (
+                self.server_side_encryption.serialize_to_json_for_request()
+            )
         result['fileRetention'] = self._file_retention_dict(account_auth_token)
         result['legalHold'] = self._legal_hold_dict(account_auth_token)
         return result
@@ -374,8 +393,9 @@ class FileSimulator:
         else:
             file_lock_configuration['value'] = {'mode': self.file_retention.mode.value}
             if self.file_retention.retain_until is not None:
-                file_lock_configuration['value']['retainUntilTimestamp'
-                                                ] = self.file_retention.retain_until
+                file_lock_configuration['value']['retainUntilTimestamp'] = (
+                    self.file_retention.retain_until
+                )
         return file_lock_configuration
 
     def _legal_hold_dict(self, account_auth_token):
@@ -423,7 +443,8 @@ class FileSimulator:
         max_part_count = max_part_count or 100
         parts = [
             part.as_list_parts_dict()
-            for part in self.parts if part is not None and start_part_number <= part.part_number
+            for part in self.parts
+            if part is not None and start_part_number <= part.part_number
         ]
         if len(parts) <= max_part_count:
             next_part_number = None
@@ -474,7 +495,7 @@ class FakeRaw:
         return self._position
 
     def read(self, size):
-        data = self.data_bytes[self._position:self._position + size]
+        data = self.data_bytes[self._position : self._position + size]
         self._position += len(data)
         return data
 
@@ -486,7 +507,7 @@ class FakeResponse:
         self.url = url
         self.range_ = range_
         if range_ is not None:
-            self.data_bytes = self.data_bytes[range_[0]:range_[1] + 1]
+            self.data_bytes = self.data_bytes[range_[0] : range_[1] + 1]
 
     @property
     def data_bytes(self):
@@ -524,7 +545,6 @@ class FakeResponse:
 
 
 class BucketSimulator:
-
     # File IDs start at 9999 and count down, so they sort in the order
     # returned by list_file_versions. The IDs are strings.
     FIRST_FILE_NUMBER = 9999
@@ -575,8 +595,14 @@ class BucketSimulator:
         self.default_retention = NO_RETENTION_BUCKET_SETTING
         self.replication = replication
         if self.replication is not None:
-            assert self.replication.asReplicationSource is None or self.replication.asReplicationSource.rules
-            assert self.replication.asReplicationDestination is None or self.replication.asReplicationDestination.sourceToDestinationKeyMapping
+            assert (
+                self.replication.asReplicationSource is None
+                or self.replication.asReplicationSource.rules
+            )
+            assert (
+                self.replication.asReplicationDestination is None
+                or self.replication.asReplicationDestination.sourceToDestinationKeyMapping
+            )
 
     def get_file(self, file_id, file_name) -> FileSimulator:
         try:
@@ -608,8 +634,9 @@ class BucketSimulator:
             default_sse['isClientAuthorizedToRead'] = True
             default_sse['value'] = {'mode': self.default_server_side_encryption.mode.value}
             if self.default_server_side_encryption.algorithm is not None:
-                default_sse['value']['algorithm'
-                                    ] = self.default_server_side_encryption.algorithm.value
+                default_sse['value']['algorithm'] = (
+                    self.default_server_side_encryption.algorithm.value
+                )
         else:
             default_sse['value'] = {'mode': EncryptionMode.UNKNOWN.value}
 
@@ -619,11 +646,13 @@ class BucketSimulator:
                 'value': {
                     'defaultRetention': {
                         'mode': self.default_retention.mode.value,
-                        'period': self.default_retention.period.as_dict() if self.default_retention.period else None,
+                        'period': self.default_retention.period.as_dict()
+                        if self.default_retention.period
+                        else None,
                     },
                     'isFileLockEnabled': self.is_file_lock_enabled,
                 },
-            }  # yapf: disable
+            }
         else:
             file_lock_configuration = {'isClientAuthorizedToRead': False, 'value': None}
 
@@ -656,8 +685,8 @@ class BucketSimulator:
             accountId=self.account_id,
             bucketId=self.bucket_id,
             fileId=file_id,
-            fileName=file_sim.name
-        )  # yapf: disable
+            fileName=file_sim.name,
+        )
 
     def delete_file_version(
         self, account_auth_token, file_id, file_name, bypass_governance: bool = False
@@ -700,8 +729,9 @@ class BucketSimulator:
         range_=None,
         encryption: EncryptionSetting | None = None,
     ):
-        files = self.list_file_names(self.api.current_token, file_name,
-                                     1)['files']  # token is not important here
+        files = self.list_file_names(self.api.current_token, file_name, 1)[
+            'files'
+        ]  # token is not important here
         if len(files) == 0:
             raise FileNotPresent(file_id_or_name=file_name)
 
@@ -736,7 +766,7 @@ class BucketSimulator:
 
     def get_file_info_by_name(self, account_auth_token, file_name):
         # Sorting files by name and ID, so lower ID (newer upload) is returned first.
-        for ((name, id), file) in sorted(self.file_name_and_id_to_file.items()):
+        for (name, id), file in sorted(self.file_name_and_id_to_file.items()):
             if file_name == name:
                 return file.as_download_headers(account_auth_token_or_none=account_auth_token)
         raise FileNotPresent(file_id_or_name=file_name, bucket_name=self.bucket_name)
@@ -744,21 +774,33 @@ class BucketSimulator:
     def get_upload_url(self, account_auth_token):
         upload_id = next(self.upload_url_counter)
         upload_url = 'https://upload.example.com/%s/%d/%s' % (
-            self.bucket_id, upload_id, account_auth_token
+            self.bucket_id,
+            upload_id,
+            account_auth_token,
         )
         return dict(bucketId=self.bucket_id, uploadUrl=upload_url, authorizationToken=upload_url)
 
     def get_upload_part_url(self, account_auth_token, file_id):
         upload_url = 'https://upload.example.com/part/%s/%d/%s' % (
-            file_id, random.randint(1, 10**9), account_auth_token
+            file_id,
+            random.randint(1, 10**9),
+            account_auth_token,
         )
         return dict(bucketId=self.bucket_id, uploadUrl=upload_url, authorizationToken=upload_url)
 
     def hide_file(self, account_auth_token, file_name):
         file_id = self._next_file_id()
         file_sim = self.FILE_SIMULATOR_CLASS(
-            self.account_id, self, file_id, 'hide', file_name, None, "none", {}, b'',
-            next(self.upload_timestamp_counter)
+            self.account_id,
+            self,
+            file_id,
+            'hide',
+            file_name,
+            None,
+            'none',
+            {},
+            b'',
+            next(self.upload_timestamp_counter),
         )
         self.file_id_to_file[file_id] = file_sim
         self.file_name_and_id_to_file[file_sim.sort_key()] = file_sim
@@ -849,14 +891,16 @@ class BucketSimulator:
             'upload',
             new_file_name,
             file_sim.content_type,
-            hex_sha1_of_bytes(data_bytes),  # we hash here again because bytes_range may not cover the full source
+            hex_sha1_of_bytes(
+                data_bytes
+            ),  # we hash here again because bytes_range may not cover the full source
             file_sim.file_info,
             data_bytes,
             next(self.upload_timestamp_counter),
             server_side_encryption=sse,
             file_retention=file_retention,
             legal_hold=legal_hold,
-        )  # yapf: disable
+        )
         destination_bucket.file_id_to_file[copy_file_sim.file_id] = copy_file_sim
         destination_bucket.file_name_and_id_to_file[copy_file_sim.sort_key()] = copy_file_sim
 
@@ -865,14 +909,14 @@ class BucketSimulator:
             copy_file_sim.file_info = file_info or file_sim.file_info
 
         ## long term storage of that file has action="upload", but here we need to return action="copy", just this once
-        #class TestFileVersionFactory(FileVersionFactory):
+        # class TestFileVersionFactory(FileVersionFactory):
         #    FILE_VERSION_CLASS = self.FILE_SIMULATOR_CLASS
 
-        #file_version_dict = copy_file_sim.as_upload_result(account_auth_token)
-        #del file_version_dict['action']
-        #print(file_version_dict)
-        #copy_file_sim_with_action_copy = TestFileVersionFactory(self.api).from_api_response(file_version_dict, force_action='copy')
-        #return copy_file_sim_with_action_copy
+        # file_version_dict = copy_file_sim.as_upload_result(account_auth_token)
+        # del file_version_dict['action']
+        # print(file_version_dict)
+        # copy_file_sim_with_action_copy = TestFileVersionFactory(self.api).from_api_response(file_version_dict, force_action='copy')
+        # return copy_file_sim_with_action_copy
 
         # TODO: the code above cannot be used right now because FileSimulator.__init__ is incompatible with FileVersionFactory / FileVersion.__init__ - refactor is needed
         # for now we'll just return the newly constructed object with a copy action...
@@ -899,8 +943,9 @@ class BucketSimulator:
         max_file_count=None,
         prefix=None,
     ):
-        assert prefix is None or start_file_name is None or start_file_name.startswith(prefix
-                                                                                      ), locals()
+        assert (
+            prefix is None or start_file_name is None or start_file_name.startswith(prefix)
+        ), locals()
         start_file_name = start_file_name or ''
         max_file_count = max_file_count or 100
         result_files = []
@@ -931,8 +976,9 @@ class BucketSimulator:
         max_file_count=None,
         prefix=None,
     ):
-        assert prefix is None or start_file_name is None or start_file_name.startswith(prefix
-                                                                                      ), locals()
+        assert (
+            prefix is None or start_file_name is None or start_file_name.startswith(prefix)
+        ), locals()
         start_file_name = start_file_name or ''
         start_file_id = start_file_id or ''
         max_file_count = max_file_count or 100
@@ -942,8 +988,8 @@ class BucketSimulator:
         for key in sorted(self.file_name_and_id_to_file):
             (file_name, file_id) = key
             if (start_file_name < file_name) or (
-                start_file_name == file_name and
-                (start_file_id == '' or int(start_file_id) <= int(file_id))
+                start_file_name == file_name
+                and (start_file_id == '' or int(start_file_id) <= int(file_id))
             ):
                 file_sim = self.file_name_and_id_to_file[key]
                 if prefix is not None and not file_name.startswith(prefix):
@@ -965,9 +1011,11 @@ class BucketSimulator:
         start_file_id = start_file_id or self.FIRST_FILE_ID
         max_file_count = max_file_count or 100
         all_unfinished_ids = set(
-            k for (k, v) in self.file_id_to_file.items()
-            if v.action == 'start' and k <= start_file_id and
-            (prefix is None or v.name.startswith(prefix))
+            k
+            for (k, v) in self.file_id_to_file.items()
+            if v.action == 'start'
+            and k <= start_file_id
+            and (prefix is None or v.name.startswith(prefix))
         )
         ids_in_order = sorted(all_unfinished_ids, reverse=True)
 
@@ -976,7 +1024,7 @@ class BucketSimulator:
             for file_sim in (
                 self.file_id_to_file[file_id] for file_id in ids_in_order[:max_file_count]
             )
-        ]  # yapf: disable
+        ]
         next_file_id = None
         if len(file_dict_list) == max_file_count:
             next_file_id = str(int(file_dict_list[-1]['fileId']) - 1)
@@ -995,7 +1043,9 @@ class BucketSimulator:
     ):
         file_id = self._next_file_id()
         sse = server_side_encryption or self.default_server_side_encryption
-        if sse:  # FIXME: remove this part when RawApi<->Encryption adapters are implemented properly
+        if (
+            sse
+        ):  # FIXME: remove this part when RawApi<->Encryption adapters are implemented properly
             file_info = sse.add_key_id_to_file_info(file_info)
 
         upload_timestamp = next(self.upload_timestamp_counter)
@@ -1003,10 +1053,20 @@ class BucketSimulator:
             upload_timestamp = custom_upload_timestamp
 
         file_sim = self.FILE_SIMULATOR_CLASS(
-            self.account_id, self, file_id, 'start', file_name, content_type, 'none',
-            file_info, None, upload_timestamp, server_side_encryption=sse,
-            file_retention=file_retention, legal_hold=legal_hold,
-        )  # yapf: disable
+            self.account_id,
+            self,
+            file_id,
+            'start',
+            file_name,
+            content_type,
+            'none',
+            file_info,
+            None,
+            upload_timestamp,
+            server_side_encryption=sse,
+            file_retention=file_retention,
+            legal_hold=legal_hold,
+        )
         self.file_id_to_file[file_id] = file_sim
         self.file_name_and_id_to_file[file_sim.sort_key()] = file_sim
         return file_sim.as_start_large_file_result(account_auth_token)
@@ -1031,8 +1091,10 @@ class BucketSimulator:
                 raise DisablingFileLockNotSupported()
 
             if (
-                not self.is_file_lock_enabled and is_file_lock_enabled and self.replication and
-                self.replication.is_source
+                not self.is_file_lock_enabled
+                and is_file_lock_enabled
+                and self.replication
+                and self.replication.is_source
             ):
                 raise SourceReplicationConflict()
 
@@ -1087,7 +1149,9 @@ class BucketSimulator:
         file_id = self._next_file_id()
 
         encryption = server_side_encryption or self.default_server_side_encryption
-        if encryption:  # FIXME: remove this part when RawApi<->Encryption adapters are implemented properly
+        if (
+            encryption
+        ):  # FIXME: remove this part when RawApi<->Encryption adapters are implemented properly
             file_info = encryption.add_key_id_to_file_info(file_info)
 
         upload_timestamp = next(self.upload_timestamp_counter)
@@ -1141,7 +1205,7 @@ class BucketSimulator:
             partNumber=part_number,
             contentLength=content_length,
             contentSha1=sha1_sum,
-        )  # yapf: disable
+        )
         if server_side_encryption is not None:
             result['serverSideEncryption'] = server_side_encryption.serialize_to_json_for_request()
         return result
@@ -1181,26 +1245,26 @@ class BucketSimulator:
     def get_notification_rules(self) -> list[NotificationRule]:
         return self._notification_rules
 
-    def set_notification_rules(self,
-                               rules: Iterable[NotificationRule]) -> list[NotificationRuleResponse]:
-        old_rules_by_name = {rule["name"]: rule for rule in self._notification_rules}
+    def set_notification_rules(
+        self, rules: Iterable[NotificationRule]
+    ) -> list[NotificationRuleResponse]:
+        old_rules_by_name = {rule['name']: rule for rule in self._notification_rules}
         new_rules: list[NotificationRuleResponse] = []
         for rule in rules:
-            for field in ("isSuspended", "suspensionReason"):
+            for field in ('isSuspended', 'suspensionReason'):
                 rule.pop(field, None)
-            old_rule = old_rules_by_name.get(rule["name"], {"targetConfiguration": {}})
+            old_rule = old_rules_by_name.get(rule['name'], {'targetConfiguration': {}})
             new_rule = {
                 **{
-                    "isSuspended": False,
-                    "suspensionReason": "",
+                    'isSuspended': False,
+                    'suspensionReason': '',
                 },
                 **old_rule,
                 **rule,
-                "targetConfiguration":
-                    {
-                        **old_rule.get("targetConfiguration", {}),
-                        **rule.get("targetConfiguration", {}),
-                    },
+                'targetConfiguration': {
+                    **old_rule.get('targetConfiguration', {}),
+                    **rule.get('targetConfiguration', {}),
+                },
             }
             new_rules.append(new_rule)
         self._notification_rules = new_rules
@@ -1210,11 +1274,11 @@ class BucketSimulator:
         self, rule_name: str, reason: str, is_suspended: bool | None = None
     ) -> None:
         for rule in self._notification_rules:
-            if rule["name"] == rule_name:
-                rule["isSuspended"] = bool(reason) if is_suspended is None else is_suspended
-                rule["suspensionReason"] = reason
+            if rule['name'] == rule_name:
+                rule['isSuspended'] = bool(reason) if is_suspended is None else is_suspended
+                rule['suspensionReason'] = reason
                 return
-        raise ResourceNotFound(f"Rule {rule_name} not found")
+        raise ResourceNotFound(f'Rule {rule_name} not found')
 
 
 class RawSimulator(AbstractRawApi):
@@ -1240,13 +1304,16 @@ class RawSimulator(AbstractRawApi):
     UPLOAD_PART_MATCHER = re.compile('https://upload.example.com/part/([^/]*)')
     UPLOAD_URL_MATCHER = re.compile(r'https://upload.example.com/([^/]*)/([^/]*)')
     DOWNLOAD_URL_MATCHER = re.compile(
-        DOWNLOAD_URL + '(?:' + '|'.join(
+        DOWNLOAD_URL
+        + '(?:'
+        + '|'.join(
             (
                 r'/b2api/v[0-9]+/b2_download_file_by_id\?fileId=(?P<file_id>[^/]+)',
                 '/file/(?P<bucket_name>[^/]+)/(?P<file_name>.+)',
             )
-        ) + ')$'
-    )  # yapf: disable
+        )
+        + ')$'
+    )
 
     def __init__(self, b2_http=None):
         # Map from application_key_id to KeySimulator.
@@ -1308,8 +1375,7 @@ class RawSimulator(AbstractRawApi):
             key=master_key,
             capabilities=ALL_CAPABILITIES,
             expiration_timestamp_or_none=None,
-            bucket_id_or_none=None,
-            bucket_name_or_none=None,
+            buckets_or_none=None,
             name_prefix_or_none=None,
         )
 
@@ -1335,21 +1401,33 @@ class RawSimulator(AbstractRawApi):
         self.current_token = auth_token
         self.auth_token_counter += 1
         self.auth_token_to_key[auth_token] = key_sim
+
         allowed = key_sim.get_allowed()
-        bucketId = allowed.get('bucketId')
-        if (bucketId is not None) and (bucketId in self.bucket_id_to_bucket):
-            allowed['bucketName'] = self.bucket_id_to_bucket[bucketId].bucket_name
-        else:
-            allowed['bucketName'] = None
+
+        buckets = allowed.get('buckets')
+        if buckets:
+            for item in buckets:
+                try:
+                    item['name'] = self.bucket_id_to_bucket[item['id']].bucket_name
+                except KeyError:
+                    item['name'] = None
+
         return dict(
             accountId=key_sim.account_id,
             authorizationToken=auth_token,
-            apiUrl=self.API_URL,
-            downloadUrl=self.DOWNLOAD_URL,
-            recommendedPartSize=self.MIN_PART_SIZE,
-            absoluteMinimumPartSize=self.MIN_PART_SIZE,
-            allowed=allowed,
-            s3ApiUrl=self.S3_API_URL,
+            apiInfo=dict(
+                groupsApi=dict(),
+                storageApi=dict(
+                    apiUrl=self.API_URL,
+                    downloadUrl=self.DOWNLOAD_URL,
+                    recommendedPartSize=self.MIN_PART_SIZE,
+                    absoluteMinimumPartSize=self.MIN_PART_SIZE,
+                    allowed=allowed,
+                    s3ApiUrl=self.S3_API_URL,
+                    capabilities=allowed['capabilities'],
+                    namePrefix=allowed['namePrefix'],
+                ),
+            ),
         )
 
     def cancel_large_file(self, api_url, account_auth_token, file_id):
@@ -1404,7 +1482,7 @@ class RawSimulator(AbstractRawApi):
         capabilities,
         key_name,
         valid_duration_seconds,
-        bucket_id,
+        bucket_ids,
         name_prefix,
     ):
         if not re.match(r'^[A-Za-z0-9-]{1,100}$', key_name):
@@ -1425,12 +1503,18 @@ class RawSimulator(AbstractRawApi):
         self.app_key_counter += 1
         application_key_id = 'appKeyId%d' % (index,)
         app_key = 'appKey%d' % (index,)
-        bucket_name_or_none = None
-        if bucket_id is not None:
+
+        buckets = None
+
+        if bucket_ids is not None:
             # It is possible for bucketId to be filled and bucketName to be empty.
             # It can happen when the bucket was deleted.
-            with suppress(NonExistentBucket):
-                bucket_name_or_none = self._get_bucket_by_id(bucket_id).bucket_name
+            buckets = []
+            for _id in bucket_ids:
+                try:
+                    buckets.append({'id': _id, 'name': self._get_bucket_by_id(_id).bucket_name})
+                except NonExistentBucket:
+                    buckets.append({'id': _id, 'name': None})
 
         key_sim = KeySimulator(
             account_id=account_id,
@@ -1439,8 +1523,7 @@ class RawSimulator(AbstractRawApi):
             key=app_key,
             capabilities=capabilities,
             expiration_timestamp_or_none=expiration_timestamp_or_none,
-            bucket_id_or_none=bucket_id,
-            bucket_name_or_none=bucket_name_or_none,
+            buckets_or_none=buckets,
             name_prefix_or_none=name_prefix,
         )
         self.key_id_to_key[application_key_id] = key_sim
@@ -1561,16 +1644,14 @@ class RawSimulator(AbstractRawApi):
         bucket = self._get_bucket_by_id(bucket_id)
         self._assert_account_auth(api_url, account_auth_token, bucket.account_id, 'shareFiles')
         return {
-            'bucketId':
+            'bucketId': bucket_id,
+            'fileNamePrefix': file_name_prefix,
+            'authorizationToken': 'fake_download_auth_token_%s_%s_%d'
+            % (
                 bucket_id,
-            'fileNamePrefix':
-                file_name_prefix,
-            'authorizationToken':
-                'fake_download_auth_token_%s_%s_%d' % (
-                    bucket_id,
-                    b2_url_encode(file_name_prefix),
-                    valid_duration_in_seconds,
-                )
+                b2_url_encode(file_name_prefix),
+                valid_duration_in_seconds,
+            ),
         }
 
     def get_file_info_by_id(self, api_url, account_auth_token, file_id):
@@ -1656,7 +1737,10 @@ class RawSimulator(AbstractRawApi):
         destination_server_side_encryption: EncryptionSetting | None = None,
         source_server_side_encryption: EncryptionSetting | None = None,
     ):
-        if destination_server_side_encryption is not None and destination_server_side_encryption.mode == EncryptionMode.SSE_B2:
+        if (
+            destination_server_side_encryption is not None
+            and destination_server_side_encryption.mode == EncryptionMode.SSE_B2
+        ):
             raise ValueError(
                 'unsupported sse mode for copy_part!'
             )  # SSE-B2 is only to be marked in b2_start_large_file
@@ -1702,7 +1786,8 @@ class RawSimulator(AbstractRawApi):
         ]
         bucket_list = [
             bucket.bucket_dict(account_auth_token)
-            for bucket in sorted_buckets if self._bucket_matches(bucket, bucket_id, bucket_name)
+            for bucket in sorted_buckets
+            if self._bucket_matches(bucket, bucket_id, bucket_name)
         ]
         return dict(buckets=bucket_list)
 
@@ -1712,9 +1797,8 @@ class RawSimulator(AbstractRawApi):
                 return bucket.bucket_id
 
     def _bucket_matches(self, bucket, bucket_id, bucket_name):
-        return (
-            (bucket_id is None or bucket.bucket_id == bucket_id) and
-            (bucket_name is None or bucket.bucket_name == bucket_name)
+        return (bucket_id is None or bucket.bucket_id == bucket_id) and (
+            bucket_name is None or bucket.bucket_name == bucket_name
         )
 
     def list_file_names(
@@ -1770,7 +1854,7 @@ class RawSimulator(AbstractRawApi):
         account_auth_token,
         account_id,
         max_key_count=1000,
-        start_application_key_id=None
+        start_application_key_id=None,
     ):
         self._assert_account_auth(api_url, account_auth_token, account_id, 'listKeys')
         next_application_key_id = None
@@ -1807,7 +1891,7 @@ class RawSimulator(AbstractRawApi):
         bucket_id,
         start_file_id=None,
         max_file_count=None,
-        prefix=None
+        prefix=None,
     ):
         bucket = self._get_bucket_by_id(bucket_id)
         self._assert_account_auth(
@@ -1864,7 +1948,15 @@ class RawSimulator(AbstractRawApi):
         replication: ReplicationConfiguration | None = None,
         is_file_lock_enabled: bool | None = None,
     ):
-        assert bucket_type or bucket_info or cors_rules or lifecycle_rules or default_server_side_encryption or replication or is_file_lock_enabled is not None
+        assert (
+            bucket_type
+            or bucket_info
+            or cors_rules
+            or lifecycle_rules
+            or default_server_side_encryption
+            or replication
+            or is_file_lock_enabled is not None
+        )
         bucket = self._get_bucket_by_id(bucket_id)
         self._assert_account_auth(api_url, account_auth_token, bucket.account_id, 'writeBuckets')
         return bucket._update_bucket(
@@ -1893,11 +1985,12 @@ class RawSimulator(AbstractRawApi):
         legal_hold: LegalHold | None,
         custom_upload_timestamp: int | None = None,
     ) -> dict:
-
         # fix to allow calculating headers on unknown key - only for simulation
-        if server_side_encryption is not None \
-           and server_side_encryption.mode == EncryptionMode.SSE_C \
-           and server_side_encryption.key.secret is None:
+        if (
+            server_side_encryption is not None
+            and server_side_encryption.mode == EncryptionMode.SSE_C
+            and server_side_encryption.key.secret is None
+        ):
             server_side_encryption.key.secret = b'secret'
 
         return super().get_upload_file_headers(
@@ -1941,7 +2034,9 @@ class RawSimulator(AbstractRawApi):
             bucket = self._get_bucket_by_id(bucket_id)
             if server_side_encryption is not None:
                 assert server_side_encryption.mode in (
-                    EncryptionMode.NONE, EncryptionMode.SSE_B2, EncryptionMode.SSE_C
+                    EncryptionMode.NONE,
+                    EncryptionMode.SSE_B2,
+                    EncryptionMode.SSE_C,
                 )
                 file_info = server_side_encryption.add_key_id_to_file_info(file_info)
 
@@ -2016,8 +2111,17 @@ class RawSimulator(AbstractRawApi):
             raise InvalidAuthToken('auth token expired', 'auth_token_expired')
         if capability not in key_sim.capabilities:
             raise Unauthorized('', 'unauthorized')
-        if key_sim.bucket_id_or_none is not None and key_sim.bucket_id_or_none != bucket_id:
-            raise Unauthorized('', 'unauthorized')
+
+        if key_sim.buckets_or_none:
+            found = False
+            for item in key_sim.buckets_or_none:
+                if item['id'] == bucket_id:
+                    found = True
+                    break
+
+            if not found:
+                raise Unauthorized('', 'unauthorized')
+
         if key_sim.name_prefix_or_none is not None:
             if file_name is not None and not file_name.startswith(key_sim.name_prefix_or_none):
                 raise Unauthorized('', 'unauthorized')
@@ -2033,8 +2137,11 @@ class RawSimulator(AbstractRawApi):
         return self.bucket_name_to_bucket[bucket_name]
 
     def set_bucket_notification_rules(
-        self, api_url: str, account_auth_token: str, bucket_id: str,
-        rules: Iterable[NotificationRule]
+        self,
+        api_url: str,
+        account_auth_token: str,
+        bucket_id: str,
+        rules: Iterable[NotificationRule],
     ):
         bucket = self._get_bucket_by_id(bucket_id)
         self._assert_account_auth(
@@ -2042,8 +2149,9 @@ class RawSimulator(AbstractRawApi):
         )
         return bucket.set_notification_rules(rules)
 
-    def get_bucket_notification_rules(self, api_url: str, account_auth_token: str,
-                                      bucket_id: str) -> list[NotificationRule]:
+    def get_bucket_notification_rules(
+        self, api_url: str, account_auth_token: str, bucket_id: str
+    ) -> list[NotificationRule]:
         bucket = self._get_bucket_by_id(bucket_id)
         self._assert_account_auth(
             api_url, account_auth_token, bucket.account_id, 'readBucketNotifications'
